# Product Context

## Why This Project Exists
Many industrial and environmental sensors use the Modbus protocol for data communication, while modern IoT and monitoring systems often rely on MQTT for data ingestion and distribution. This project bridges the gap between legacy Modbus devices and MQTT-based platforms, enabling seamless integration and real-time data flow.

## Problems Solved
- Eliminates the need for manual data collection from Modbus sensors.
- Automates the process of converting and forwarding sensor data to cloud or local MQTT brokers.
- Provides a reliable, scriptable, and extensible solution for integrating Modbus sensors into IoT ecosystems.

## How It Should Work
- The script runs continuously, connecting to a Modbus TCP device to read temperature and humidity data at regular intervals.
- Data is processed and formatted as JSON, then published to a specified MQTT topic.
- The system handles connection issues gracefully, with logging for troubleshooting.

## User Experience Goals
- Simple configuration via code variables (host, port, credentials, topic, etc.).
- Clear, timestamped logging for all major events and errors.
- Data is published in a structured, readable JSON format for easy consumption by MQTT subscribers.
- Minimal setup required; designed for headless/server environments. 